package com.gic.cloud.operation.web.controller;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.authcenter.security.core.dto.UserDetailsVO;
import com.gic.authcenter.security.core.util.UserUtils;
import com.gic.cloud.constants.IndexTypeEnum;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.dto.IndexLogDTO;
import com.gic.cloud.operation.web.vo.IndexVO;
import com.gic.cloud.qo.IndexQO;
import com.gic.cloud.service.IndexApiService;
import com.gic.cloud.service.IndexLogApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.redis.data.util.RedisUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-09 14:22
 */
@RestController
public class IndexController {

    @Autowired
    private IndexApiService indexApiService;
    @Autowired
    private IndexLogApiService indexLogApiService;


    @RequestMapping("/save-index")
    public RestResponse saveIndex(IndexQO indexQO, String relModuleIds) {
        IndexDTO indexDTO = EntityUtil.changeEntityByJSON(IndexDTO.class, indexQO);
        if (indexDTO.getClassifyType() == null ||
                StringUtils.isBlank(indexDTO.getCode()) ||
                StringUtils.isBlank(indexDTO.getName()) ||
                StringUtils.isBlank(indexDTO.getIndexRemark())) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        UserDetailsVO user = UserUtils.getUser();
        indexDTO.setClassifyName(IndexTypeEnum.parse(indexDTO.getClassifyType()));
        indexDTO.setOptUserId(user.getId());
        indexDTO.setOptUserName(user.getRealName());
        if (StringUtils.isNotBlank(relModuleIds)) {
            indexDTO.setRelModuleIdList(Lists.newArrayList(relModuleIds.split(",")));
        }
        ServiceResponse<Void> serviceResponse = indexApiService.saveIndex(indexDTO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/update-index")
    public RestResponse updateIndex(IndexDTO indexDTO, String relModuleIds, Integer updateType, String reason) {
        if (indexDTO.getIndexId() == null || updateType == null) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        Integer id = UserUtils.getUser().getId();
        String realName = UserUtils.getUser().getRealName();
        indexDTO.setOptUserId(id);
        indexDTO.setOptUserName(realName);
        if (StringUtils.isNotBlank(relModuleIds)) {
            indexDTO.setRelModuleIdList(Lists.newArrayList(relModuleIds.split(",")));
        }
        ServiceResponse<Void> serviceResponse = indexApiService.updateIndex(indexDTO, updateType, reason);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/get-by-index-id")
    public RestResponse getByIndexId(Integer indexId) {
        ServiceResponse<IndexDTO> serviceResponse = indexApiService.getByIndexId(indexId);
        return ResultControllerUtils.commonResultOne(serviceResponse, IndexVO.class);
    }

    @RequestMapping("/list-index")
    public RestResponse listIndex(IndexQO indexQO) {
        ServiceResponse<Page<IndexDTO>> serviceResponse = indexApiService.listIndex(indexQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-all-index")
    public RestResponse listAllIndex(IndexQO indexQO) {
        ServiceResponse<List<IndexDTO>> serviceResponse = indexApiService.listAllIndex(indexQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/index-log")
    public RestResponse indexLog(Integer indexId) {
        ServiceResponse<List<IndexLogDTO>> serviceResponse = indexLogApiService.listIndexLog(indexId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/delete-index")
    public RestResponse delete(Integer indexId) {
        ServiceResponse<Void> serviceResponse = indexApiService.delete(indexId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/publish-index")
    public RestResponse publishIndex(String indexIds, Integer updateType, String reason) {
        if (StringUtils.isBlank(indexIds)) {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), ErrorCode.PARAMETER_ERROR.getMsg());
        }
        Integer id = UserUtils.getUser().getId();
        String realName = UserUtils.getUser().getRealName();
        List<Integer> indexIdList;
        if ("all".equals(indexIds)) {
            List<IndexDTO> list = indexApiService.listAllIndex(new IndexQO()).getResult();
            indexIdList = list.stream().map(IndexDTO::getIndexId).collect(Collectors.toList());
        } else {
            indexIdList = Stream.of(indexIds.split(",")).map(Integer::valueOf).collect(Collectors.toList());
        }
        ServiceResponse<Void> serviceResponse = indexApiService.publish(indexIdList, updateType, reason, id, realName);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/module-list")
    public RestResponse moduleList() {
        Object cache = RedisUtil.getCache("enterprise:datacloud:moduleList");
        if (cache != null) {
            return RestResponse.success(JSON.parseArray(cache.toString()));
        }
        return RestResponse.success();
    }

}