package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.dto.IndexDescDTO;
import com.gic.cloud.entity.TabIndex;
import com.gic.cloud.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-23 15:10
 */
@Service("indexDescApiService")
public class IndexDescApiServiceImpl implements IndexDescApiService {

    @Autowired
    private IndexDescService indexDescService;
    @Autowired
    private IndexDescEnterpriseService indexDescEnterpriseService;
    @Autowired
    private IndexService indexService;
    @Autowired
    private UpdateTipService updateTipService;

    @Override
    public ServiceResponse<List<IndexDescDTO>> listByEnterpriseAndModule(Integer enterpriseId, String moduleId, String indexGroupName) {
        List<IndexDescDTO> descList = indexDescService.listByEnterpriseAndModule(enterpriseId, moduleId, indexGroupName);
        return EnterpriseServiceResponse.success(descList);
    }

    @Override
    public ServiceResponse<List<IndexDescDTO>> listByEnterpriseAndModuleDetail(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId) {
        List<IndexDescDTO> descList = indexDescService.listByEnterpriseAndModuleDetail(enterpriseId, moduleId, indexGroupName, userId);
        for (int i = 0; i < descList.size(); i++) {
            descList.get(i).setSeq(i + 1);
        }
        return EnterpriseServiceResponse.success(descList);
    }

    @Override
    public ServiceResponse<List<IndexDescDTO>> listRecycle(Integer enterpriseId, String moduleId, String indexGroupName, Integer userId) {
        List<IndexDescDTO> list = indexDescService.listRecycle(enterpriseId, moduleId, indexGroupName, userId);
        return EnterpriseServiceResponse.success(list);
    }

    @Override
    public ServiceResponse<Void> sort(Integer enterpriseId, String moduleId, String indexGroupName, Integer indexDescEnterpriseId, Integer seq) {
        indexDescService.sort(enterpriseId, moduleId, indexGroupName, indexDescEnterpriseId, seq);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> moveToRecycle(Integer indexDescEnterpriseId) {
        indexDescEnterpriseService.moveToRecycle(indexDescEnterpriseId);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexDTO> getIndexDesc(Integer enterpriseId, String moduleId, String indexGroupName, String indexCode) {
        TabIndex index = indexService.getByIndexCode(moduleId, indexCode);
        return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(IndexDTO.class, index));
    }

    @Override
    public ServiceResponse<Void> batchUpdate(List<IndexDescDTO> descList) {
        indexDescEnterpriseService.batchUpdate(descList);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> batchUpdate(Integer enterpriseId, String moduleId, String ids) {
        List<IndexDescDTO> indexDescDTOList = indexDescService.listByEnterpriseAndModuleDetail(enterpriseId, moduleId, null, null);
        String[] idsArr = ids.split(",");
        indexDescEnterpriseService.batchUpdate(idsArr);
        List<String> existList = Arrays.asList(idsArr);
        if (existList.size() < indexDescDTOList.size()) {
            List<Integer> deleteList = indexDescDTOList.stream().map(IndexDescDTO::getIndexDescEnterpriseId).filter(e -> !existList.contains(e.toString())).collect(Collectors.toList());
            indexDescEnterpriseService.deleteList(deleteList);
        }
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> eraseUpdateTips(Integer userId, Integer updateTipsId) {
        updateTipService.erase(userId, updateTipsId, 1);
        return EnterpriseServiceResponse.success();
    }
}
